/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package edu.upennlib.sushitoolkit
import org.apache.commons.httpclient.UsernamePasswordCredentials
import org.apache.commons.httpclient.auth.AuthScope
import org.apache.commons.httpclient.HttpClient
import org.apache.commons.httpclient.cookie.CookiePolicy
import org.apache.commons.httpclient.methods.PostMethod
import org.apache.commons.httpclient.methods.StringRequestEntity
import org.apache.commons.io.IOUtils
import org.apache.log4j.Logger
import org.apache.log4j.Level
import org.apache.log4j.PropertyConfigurator
import groovy.text.SimpleTemplateEngine
import java.text.SimpleDateFormat
import groovy.grape.Grape
import org.apache.commons.lang.Validate

/*
 * Beginning of the command line call
 */

def cli = new CliBuilder (usage:'groovy sushiCall.groovy [-<p|d>] <property file or directory>')
cli.with {
     h longOpt:'help', 'Usage information'
     p longOpt:'properties', 'property file to use', args:1
     r longOpt:'print', 'prints the response, this is the default'
     f longOpt:'toFile', 'adds the response to a file', args:1
}

def opts = cli.parse(args)
configureLogger()
run(opts)

/*
 * End of the command line call
 */


@Grapes([
    @Grab(group='commons-httpclient', module='commons-httpclient', version='3.1'),
    @Grab(group='commons-lang', module='commons-lang', version='2.5'),
    @Grab(group='commons-io', module='commons-io', version='1.4'),
    @Grab(group='log4j', module='log4j', version='1.2.16')
])
def run(opts) {
    if (opts.h) {
        cli.usage()
        System.exit(0)
    }

    def overloadProperties = new Properties();
    if (opts.p) {
        overloadProperties.load(new FileInputStream(new File(opts.p)))
    }

    def sushiRequest = new SushiRequest()
    def runner = new SushiCallRunner()
    

    if (opts.r || !opts.f) {
        println runner.getResponseText(sushiRequest, overloadProperties)
    } else {
        def responseStream = runner.getResponseStream(sushiRequest, overloadProperties)
        def fileOut = new FileOutputStream(new File(opts.f))
        IOUtils.copy(responseStream, fileOut)
    }
}

def configureLogger() {
    def properties = new Properties()
    properties.setProperty("log4j.rootLogger", "INFO, A1")
    properties.setProperty("log4j.appender.A1", "org.apache.log4j.ConsoleAppender")
    properties.setProperty("log4j.appender.A1.layout", "org.apache.log4j.PatternLayout")
    properties.setProperty("log4j.appender.A1.layout.ConversionPattern", "%d [%t] %-5p %c - %m%n")
    PropertyConfigurator.configure(properties)
}

class SushiCallRunner {

    def HttpService httpService = new HttpServiceImpl()
    def Logger logger = Logger.getLogger(SushiCallRunner.class);

    public InputStream getResponseStream(SushiRequest sushiRequest, Properties properties) {
        loadProperties(sushiRequest, properties)
        httpService.execute(sushiRequest)
    }

    public String getResponseText(SushiRequest sushiRequest, Properties properties) {
        return IOUtils.toString(getResponseStream(sushiRequest, properties))
    }

    protected void loadProperties(SushiRequest sushiRequest, Properties overloadProperties) {

        def propSet = getPropertySet(sushiRequest)

        propSet.each{
            if (overloadProperties.containsKey(it)) {
                logger.info "overloading $it from overload properties"
                sushiRequest."$it" = overloadProperties.get(it)
            }
            if (System.properties.containsKey(it)) {
                logger.info "overloading $it from system property"
                sushiRequest."$it" = System.properties.get(it)
            }
        }
    }

    protected void loadLibraries(boolean isCompiled) {
        if (!isCompiled) {
            Grape.grab(group:'commons-io', module:'commons-io', version:'1.4')
            Grape.grab(group:'commons-httpclient', module:'commons-httpclient', version:'3.1')
            Grape.grab(group:'commons-lang', module:'commons-lang', version:'2.5')
        }
    }

    protected HashSet<String> getPropertySet(SushiRequest sushiRequest) {
        def propSet = new HashSet<String>();

        sushiRequest.metaClass.properties.each {
            propSet.add(it.name)
        }

        return propSet
    }

    protected String getBaseUri(String path) {
        def index = path.lastIndexOf("/")
        return path.substring(0, index + 1);
    }
}

public static interface HttpService {
    InputStream execute(SushiRequest sushiRequest)
}

public static class HttpServiceImpl implements HttpService{

    def Logger logger = Logger.getLogger(HttpServiceImpl.class);

    def InputStream execute(SushiRequest sushiRequest) {
        def client = new HttpClient()
        def post = new PostMethod(sushiRequest.sushiUrl)
        post.requestEntity = new StringRequestEntity(sushiRequest.soap, "text/xml", "utf-8")
        post.setRequestHeader("SOAPAction", sushiRequest.soapAction)

        
        if (sushiRequest.basicUsername && sushiRequest.basicPassword) {
            logger.info "using http basic authentication"
            client.state.setCredentials(AuthScope.ANY, new UsernamePasswordCredentials(sushiRequest.basicUsername,
                    sushiRequest.basicPassword))
            client.params.authenticationPreemptive = true
            post.doAuthentication = true
        }

        client.executeMethod(post)
        return post.responseBodyAsStream
    }
}



/*
 * object representation of a sushi call.  Defaults to project euclid call.
*/
public static class SushiRequest {

    private static final String DEFAULT = "NA"
    def reportRequestId = DEFAULT
    def dateFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss")
    def created = dateFormatter.format(new Date())
    def requestorId = "demo"
    def requestorName = DEFAULT
    def requestorEmail = DEFAULT
    def customerId = "demo"
    def customerName = DEFAULT
    def reportName = "JR1"
    def reportRelease = 3
    def beginDate = "2008-01-01";
    def endDate = "2009-12-31";
    def sushiUrl = "http://projecteuclid.org/sushi/16/service.cgi"
    def basicUsername
    def basicPassword
    def sushiProviderName
    def soapAction = "SushiService:GetReportIn"
    def soapHeader = ""

    String getSoap() {
        def binding = [reportRequestId: reportRequestId,
                        created: created,
                        requestorId: requestorId,
                        requestorName: requestorName,
                        requestorEmail: requestorEmail,
                        customerId: customerId,
                        soapHeader:soapHeader,
                        customerName: customerName,
                        reportName: reportName,
                        reportRelease: reportRelease,
                        beginDate: beginDate,
                        endDate:endDate,
                    ]

        def soapTemplate = '''
            <soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
                  xmlns:coun="http://www.niso.org/schemas/sushi/counter"
                  xmlns:sus="http://www.niso.org/schemas/sushi">

                <soapenv:Header>${soapHeader}</soapenv:Header>
                <soapenv:Body>
                    <coun:ReportRequest Created="${created}" ID="${reportRequestId}">
                        <sus:Requestor>
                            <sus:ID>${requestorId}</sus:ID>
                            <sus:Name>${requestorName}</sus:Name>
                            <sus:Email>${requestorEmail}</sus:Email>
                        </sus:Requestor>
                        <sus:CustomerReference>
                            <sus:ID>${customerId}</sus:ID>
                            <!--Optional:-->
                            <sus:Name>${customerName}</sus:Name>
                        </sus:CustomerReference>
                        <sus:ReportDefinition Name="${reportName}" Release="${reportRelease}">
                            <sus:Filters>
                                <sus:UsageDateRange>
                                    <sus:Begin>${beginDate}</sus:Begin>
                                    <sus:End>${endDate}</sus:End>
                                </sus:UsageDateRange>
                            </sus:Filters>
                        </sus:ReportDefinition>
                    </coun:ReportRequest>
                </soapenv:Body>
            </soapenv:Envelope>
        '''

        def engine = new SimpleTemplateEngine()
        engine.createTemplate(soapTemplate).make(binding).toString()
    }
}